#!/usr/bin/perl
# $Id: image_extractor.pl,v 1.3 2013/05/17 12:38:17 rp Exp $
#
# PDF image extractor based on PDFlib TET
#

use PDFlib::TET 4.0;
use strict;


# global option list 
my $globaloptlist = "searchpath={{../data}}";

# document-specific option list 
my $docoptlist = "";

# page-specific option list 
my $pageoptlist = "skipengines={text}";

# here you can insert basic image extract options (more below) 
my $baseimageoptlist = "";

my $pageno = 0;
eval  {
    my $tet;

    if ($#ARGV != 0) {
	die("usage: image_extractor.pl <infilename>\n");
    }

    $tet = new PDFlib::TET;

    my $n_pages;
    my $doc;
    my $outfilebase;

    $outfilebase = $ARGV[0];

    $tet->set_option($globaloptlist);

    $doc = $tet->open_document($ARGV[0], $docoptlist);

    if ($doc == -1) {
	die("Error ". $tet->get_errnum() . " in " . $tet->get_apiname()
	    . "(): " . $tet->get_errmsg() . "\n");
    }

    # get number of pages in the document 
    $n_pages = $tet->pcos_get_number($doc, "length:pages");

    # loop over pages in the document 
    for ($pageno = 1; $pageno <= $n_pages; ++$pageno)
    {
	my $page;
	my $len;

	$page = $tet->open_page($doc, $pageno, $pageoptlist);

	if ($page == -1) {
	    print("Error ". $tet->get_errnum() ." in ". $tet->get_apiname()
		. "(): " . $tet->get_errmsg() . "\n");
	    next;                        # try next page 
	}

        # Retrieve all images on the page 
        while (my $ti = $tet->get_image_info($page) ) {
           # Report image geometry 
            print("page " . $pageno . ": " .
                $ti->{width} . "x" .
                $ti->{height} . "pt, alpha=" .
                $ti->{alpha} . ", beta=" .
                $ti->{beta} . "\n");

            # Retrieve additional image properties with pCOS 
            print("   id=" . $ti->{imageid} . ", " .
                $tet->pcos_get_number($doc, "images[" . $ti->{imageid} . "]/Width"). "x" .
                $tet->pcos_get_number($doc, "images[" . $ti->{imageid} . "]/Height") . " pixel, ");
            my $cs =  $tet->pcos_get_number($doc, "images[". $ti->{imageid} .
                "]/colorspaceid");

            if ($cs != -1){
            print( $tet->pcos_get_number($doc, "colorspaces[" . $cs . "]/components") . "x" .
                $tet->pcos_get_number($doc, "images[" . $ti->{imageid} . "]/bpc") . " bit " .
                $tet->pcos_get_string($doc, "colorspaces[" . $cs . "]/name") . "\n");
            }
            else
            {
                # cs==-1 may happen for some JPEG 2000 images. bpc,
                # colorspace name and number of components are not
                # available in this case.
                 

                print("JPEG2000\n");
            }
            #
            # Fetch image data and write it to a disk file. The
            # output filename is generated from the input filename,
            # page number and image ID.
             
            my $imageoptlist = $baseimageoptlist . " filename {" .
                $outfilebase . "_p" . $pageno . "_I" . $ti->{imageid} . "}";
            if ($tet->write_image_file($doc, $ti->{imageid}, $imageoptlist) == -1){
                print("Error " . $tet->get_errnum() . " in " .
                    $tet->get_apiname() . "(): " . $tet->get_errmsg() . "\n");
            }

	}

	if ($tet->get_errnum() != 0) {
	    print("Error ". $tet->get_errnum() . " in " . 
		    $tet->get_apiname() . "(): on page $pageno" 
		    . $tet->get_errmsg() . "\n");
	}

	$tet->close_page($page);
    }

    $tet->close_document($doc);
};

if ($@) {
    printf("PDFlib Exception occurred:\n");
    if ($pageno == 0) {
	printf("Error $@\n");
    } else {
	printf("Error $@ on page $pageno\n");
    }
    exit;
}
