#!/usr/bin/env ruby
# Id: extractor.rb,v 1.1 2011/11/29 15:43:26 rjs Exp 
#
# PDF image extractor based on PDFlib TET
#

require './TET'


# global option list
globaloptlist = "searchpath={{../data}}"

# document-specific option list
docoptlist = ""

# page-specific option list
pageoptlist = "skipengines={text}"

# here you can insert basic image extract options (more below) 
baseimageoptlist = ""

pageno = 0

begin
    if (ARGV.length != 1)
	raise "usage: image_extractor.pl <infilename>"
    end

    outfilebase = ARGV[0]


    tet = TET.new

    tet.set_option(globaloptlist)

    doc = tet.open_document(ARGV[0], docoptlist)

    if (doc == -1)
	raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname() \
	    + "(): " + tet.get_errmsg()
    end

    # get number of pages in the document
    n_pages = tet.pcos_get_number(doc, "length:pages")

    # loop over pages in the document
    1.upto(n_pages) do |pageno|
	page = tet.open_page(doc, pageno, pageoptlist)

	if (page == -1) 
	    print "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname()\
		+ "(): " + tet.get_errmsg()
	    next                        # try next page
	end

	# Retrieve all text fragments This is actually not required
	# for granularity=page, but must be used for other granularities.
	while (ti = tet.get_image_info(page))
	    # Report image geometry 
	    print "page " + pageno.to_s + ": " + 
		ti.width.to_s + "x" + 
		ti.height.to_s + "pt, alpha=" + 
		ti.alpha.to_s + ", beta=" + 
		ti.beta.to_s

	    # Retrieve additional image properties with pCOS 
	    print "   id=" + ti.imageid.to_s + ", " +
		tet.pcos_get_number(doc, "images[" + ti.imageid.to_s + 
		    "]/Width").to_s+ "x" + 
		tet.pcos_get_number(doc, "images[" + ti.imageid.to_s + 
		    "]/Height").to_s+ " pixel, " 
	    cs =  tet.pcos_get_number(doc, "images["+ ti.imageid.to_s + 
		"]/colorspaceid")

	    if (cs != -1)
		print tet.pcos_get_number(doc, "colorspaces[" + cs.to_i.to_s + 
			"]/components").to_s + "x" + \
		    tet.pcos_get_number(doc, "images[" + ti.imageid.to_s + 
			"]/bpc").to_s + " bit " + \
		tet.pcos_get_string(doc, "colorspaces[" + cs.to_i.to_s + 
			"]/name") + "\n"
	    else
		# cs==-1 may happen for some JPEG 2000 images. bpc,
		# colorspace name and number of components are not
		# available in this case.
		 

		print "JPEG2000\n"
	    end
	    #
	    # Fetch image data and write it to a disk file. The
	    # output filename is generated from the input filename,
	    # page number and image ID.
	     
	    imageoptlist = baseimageoptlist + " filename {" +
		outfilebase + "_p" + pageno.to_s + "_I" + ti.imageid.to_s + "}"
	    if (tet.write_image_file(doc, ti.imageid, imageoptlist) == -1)
		print "Error " + tet.get_errnum().to_s + " in " +
		    tet.get_apiname() + "(): " + tet.get_errmsg()
	    end

         

	end

	if (tet.get_errnum() != 0)
	    raise "Error " + tet.get_errnum().to_s + " in " + tet.get_apiname()\
		+ "(): on page pageno " + tet.get_errmsg()
	end

	tet.close_page(page)
    end

    tet.close_document(doc)

rescue TETException => pe
    print pe.backtrace.join("\n") + "\n"
    print "Error [" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
	                        ": " + pe.get_errmsg
    print " on page pageno" if (pageno != 0)
    print "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    tet.delete() if tet
end
